/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.lagache.spherevtk;

import org.bioimageanalysis.icy.gui.canvas.IcyCanvas;
import org.bioimageanalysis.icy.gui.canvas.VtkCanvas;
import org.bioimageanalysis.icy.model.overlay.Overlay;
import org.bioimageanalysis.icy.model.overlay.VtkPainter;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import vtk.*;

import java.awt.*;

/**
 * @author Stephane Dallongeville
 */
public class SphereVTK extends Overlay implements VtkPainter {
    double[] x, y, z;
    double size;
    Color color;

    private vtkActor[] sphere_ens;

    public SphereVTK(final double[] x, final double[] y, final double[] z, final double size, final Color color, final String name) {
        super(name);

        this.x = x;
        this.y = y;
        this.z = z;
        this.size = size;
        this.color = color;

        init();
    }

    private void init() {
        // create points
        final vtkPoints points = new vtkPoints();
        // setup scales
        final vtkFloatArray scales = new vtkFloatArray();
        scales.SetName("scales");
        // setup color label
        final vtkFloatArray col = new vtkFloatArray();
        col.SetName("col");

        // setup lookupTable and add some colors
        final vtkLookupTable colors = new vtkLookupTable();
        colors.SetNumberOfTableValues(1);
        // index, R, G, B, A
        colors.SetTableValue(0, color.getRed() / 255f, color.getGreen() / 255f, color.getBlue() / 255f, 1.0);

        for (int i = 0; i < x.length; i++) {
            points.InsertNextPoint(x[i], y[i], z[i]);
            scales.InsertNextValue(1);
            col.InsertNextValue(0); // first color
        }

        // grid structured to append center, radius and color label
        final vtkUnstructuredGrid grid = new vtkUnstructuredGrid();

        grid.SetPoints(points);
        grid.GetPointData().AddArray(scales);
        grid.GetPointData().SetActiveScalars("scales"); // !!!to set radius first
        grid.GetPointData().AddArray(col);

        points.Delete();
        scales.Delete();
        col.Delete();

        // create anything you want here, we will use a sphere for the demo
        final vtkSphereSource sphere = new vtkSphereSource();

        sphere.SetRadius(size);
        sphere.SetThetaResolution(6);
        sphere.SetPhiResolution(6);

        // object to group sphere and grid and keep smooth interaction
        final vtkGlyph3D glyph3D = new vtkGlyph3D();

        glyph3D.SetInputData(grid);
        glyph3D.SetSourceConnection(sphere.GetOutputPort());

        // create a mapper and actor
        final vtkPolyDataMapper mapper = new vtkPolyDataMapper();
        mapper.SetInputConnection(glyph3D.GetOutputPort());

        mapper.SetScalarModeToUsePointFieldData(); // without, color are displayed regarding radius and not color label
        // mapper.SetScalarRange(0, 3); // to scale color label (without, col should be between 0 and 1)
        mapper.SelectColorArray("col"); // !!!to set color (nevertheless you will have nothing)
        mapper.SetLookupTable(colors);

        final vtkActor actor = new vtkActor();
        actor.SetMapper(mapper);

        sphere_ens = new vtkActor[1];
        sphere_ens[0] = actor;
    }

    @Override
    public void paint(final Graphics2D g, final Sequence sequence, final IcyCanvas canvas) {
        super.paint(g, sequence, canvas);

        if (canvas instanceof VtkCanvas) {
            // disable picking by using this property
            ((VtkCanvas) canvas).getPicker().PickFromListOn();
        }
    }

    @Override
    public vtkProp[] getProps() {
        return sphere_ens;
    }
}